@@ -50,7 +50,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 2009-2011 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009-2015 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
@@ -86,61 +86,51 @@
  * only if the new code is made subject to such option by the copyright
  * holder.
  *
- * Portions Copyright 2012 OPEN-XCHANGE, licensed under GPL Version 2.
+ * Portions Copyright 2016-2020 OX Software GmbH, licensed under GPL Version 2.
  */
 
 package com.openexchange.http.grizzly.service.http;
 
+import org.glassfish.grizzly.servlet.FilterRegistration;
+import org.osgi.framework.Bundle;
+import org.osgi.service.http.HttpContext;
+import org.osgi.service.http.HttpService;
+import org.osgi.service.http.NamespaceException;
+import com.openexchange.exception.ExceptionUtils;
+import com.openexchange.log.LogProperties;
+import com.openexchange.marker.OXThreadMarker;
+import javax.servlet.Filter;
+import javax.servlet.Servlet;
+import javax.servlet.ServletException;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.nio.CharBuffer;
 import java.nio.charset.Charset;
 import java.nio.charset.CharsetEncoder;
-import java.util.ArrayList;
-import java.util.Collections;
-import java.util.Dictionary;
-import java.util.Enumeration;
-import java.util.HashMap;
-import java.util.LinkedList;
-import java.util.List;
-import java.util.Map.Entry;
-import java.util.Set;
-import java.util.TreeSet;
+import java.util.*;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.locks.ReentrantLock;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
 import java.util.concurrent.locks.ReentrantReadWriteLock.ReadLock;
-
-import javax.servlet.Filter;
-import javax.servlet.Servlet;
-import javax.servlet.ServletException;
-
+import org.glassfish.grizzly.http.server.ErrorPageGenerator;
 import org.glassfish.grizzly.http.server.HttpHandler;
+import org.glassfish.grizzly.http.server.OXErrorPageGenerator;
 import org.glassfish.grizzly.http.server.Request;
 import org.glassfish.grizzly.http.server.Response;
-import org.glassfish.grizzly.http.server.io.OutputBuffer;
 import org.glassfish.grizzly.http.server.util.MappingData;
 import org.glassfish.grizzly.http.util.HttpStatus;
-import org.osgi.framework.Bundle;
-import org.osgi.service.http.HttpContext;
-import org.osgi.service.http.HttpService;
-import org.osgi.service.http.NamespaceException;
-
-import com.openexchange.exception.OXException;
-import com.openexchange.http.grizzly.servletfilter.RequestReportingFilter;
-import com.openexchange.http.grizzly.servletfilter.WrappingFilter;
-import com.openexchange.log.LogProperties;
-import com.openexchange.tools.exceptions.ExceptionUtils;
 
 /**
  * OSGi Main HttpHandler.
  * <p/>
- * Dispatching HttpHandler. Grizzly integration.
+ * Dispatching HttpHandler.
+ * Grizzly integration.
  * <p/>
  * Responsibilities:
  * <ul>
  * <li>Manages registration data.</li>
- * <li>Dispatching {@link HttpHandler#service(Request, Response)} method call to registered {@link HttpHandler}s.</li>
+ * <li>Dispatching {@link HttpHandler#service(Request, Response)} method call to registered
+ * {@link HttpHandler}s.</li>
  * </ul>
  *
  * @author Hubert Iwaniuk
@@ -152,10 +142,6 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
 
     private static final AtomicBoolean SHUTDOWN_REQUESTED = new AtomicBoolean(false);
 
-    private static CharBuffer reponseBuffer = CharBuffer.allocate(4096);
-
-    private static CharsetEncoder encoder = Charset.forName("UTF-8").newEncoder();
-
     /**
      * Sets the "shut-down requested" marker.
      */
@@ -170,24 +156,26 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
         SHUTDOWN_REQUESTED.set(false);
     }
 
-    // -------------------------------------------------------------------------------------------------------------------------
-
     private final ReentrantReadWriteLock lock = new ReentrantReadWriteLock();
     private final Bundle bundle;
+    private final List<FilterAndPath> initialFilters;
     private final OSGiCleanMapper mapper;
     private final HttpStatus shutDownStatus;
+    private final ErrorPageGenerator errorPageGenerator;
 
     /**
      * Constructor.
      *
+     * @param initialFilters The initial Servlet filter to apply
      * @param bundle Bundle that we create if for, for local data reference.
      */
-    public OSGiMainHandler(Bundle bundle) {
+    public OSGiMainHandler(List<FilterAndPath> initialFilters, Bundle bundle) {
         super();
+        this.initialFilters = initialFilters;
         this.bundle = bundle;
         this.mapper = new OSGiCleanMapper();
         this.shutDownStatus = HttpStatus.newHttpStatus(HttpStatus.SERVICE_UNAVAILABLE_503.getStatusCode(), "Server shutting down...");
-        ServletFilterRegistration.getInstance().setOSGiMainHandler(this);
+        errorPageGenerator = OXErrorPageGenerator.getInstance();
     }
 
     /**
@@ -229,7 +217,13 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
                 try {
                     updateMappingInfo(request, alias, originalAlias);
 
-                    httpHandler.service(request, response);
+                    OXThreadMarker threadMarker = threadMarker();
+                    threadMarker.setHttpRequestProcessing(true);
+                    try {
+                        httpHandler.service(request, response);
+                    } finally {
+                        threadMarker.setHttpRequestProcessing(false);
+                    }
                 } catch (Throwable t) {
                     ExceptionUtils.handleThrowable(t);
                     StringBuilder logBuilder = new StringBuilder(128).append("Error processing request:\n");
@@ -247,7 +241,7 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
                 } else if ("/".equals(alias)) {
                     // 404 in "/", cutoff algo will not escape this one.
                     break;
-                } else if (!cutOff) {
+                } else if (!cutOff){
                     // not found and haven't run in cutoff mode
                     cutOff = true;
                 }
@@ -256,7 +250,7 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
         if (!invoked) {
             response.setStatus(HttpStatus.NOT_FOUND_404);
             try {
-                customizedErrorPage(request, response);
+                writeErrorPage(request, response);
             } catch (Exception e) {
                 LOG.warn("Failed to commit 404 status.", e);
             }
@@ -279,19 +273,24 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
     /**
      * Registers {@link services.http.OSGiServletHandler} in OSGi Http Service.
      * <p/>
-     * Keeps track of all registrations, takes care of thread safety.
+     * Keeps truck of all registrations, takes care of thread safety.
      *
-     * @param alias Alias to register, if wrong value than throws {@link org.osgi.service.http.NamespaceException}.
-     * @param servlet Servlet to register under alias, if fails to {@link javax.servlet.Servlet#init(javax.servlet.ServletConfig)} throws
-     *            {@link javax.servlet.ServletException}.
-     * @param initparams Initial parameters to populate {@link javax.servlet.ServletContext} with.
-     * @param context OSGi {@link org.osgi.service.http.HttpContext}, provides mime handling, security and bundle specific resource access.
+     * @param alias       Alias to register, if wrong value than throws {@link org.osgi.service.http.NamespaceException}.
+     * @param servlet     Servlet to register under alias, if fails to {@link javax.servlet.Servlet#init(javax.servlet.ServletConfig)}
+     *                    throws {@link javax.servlet.ServletException}.
+     * @param initparams  Initial parameters to populate {@link javax.servlet.ServletContext} with.
+     * @param context     OSGi {@link org.osgi.service.http.HttpContext}, provides mime handling, security and bundle specific resource access.
      * @param httpService Used to {@link HttpService#createDefaultHttpContext()} if needed.
-     * @throws org.osgi.service.http.NamespaceException If alias was invalid or already registered.
+     * @throws org.osgi.service.http.NamespaceException
+     *                                        If alias was invalid or already registered.
      * @throws javax.servlet.ServletException If {@link javax.servlet.Servlet#init(javax.servlet.ServletConfig)} fails.
-     * @throws OXException
      */
-    public void registerServletHandler(final String alias, Servlet servlet, Dictionary initparams, HttpContext context, HttpService httpService) throws NamespaceException, ServletException {
+    public void registerServletHandler(final String alias,
+                                       final Servlet servlet,
+                                       final Dictionary initparams,
+                                       HttpContext context,
+                                       final HttpService httpService)
+            throws NamespaceException, ServletException {
 
         ReentrantLock lock = OSGiCleanMapper.getLock();
         lock.lock();
@@ -312,16 +311,19 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
              */
             if (context == null) {
                 LOG.debug("No HttpContext provided, creating default");
-                context = httpService.createDefaultHttpContext();
+                context = null == httpService ? new HttpContextImpl(bundle) : httpService.createDefaultHttpContext();
             }
-            OSGiServletHandler servletHandler = findOrCreateOSGiServletHandler(servlet, context, initparams);
+
+            OSGiServletHandler servletHandler =
+                    findOrCreateOSGiServletHandler(servlet, context, initparams);
             servletHandler.setServletPath(alias);
-            addServletFilters(servletHandler);
+            addInitialServletFilters(context);
 
             /*
              * Servlet would be started several times if registered with multiple aliases. Starting means: 1. Set ContextPath 2. Instantiate
              * Servlet if null 3. Call init(config) on the Servlet.
              */
+            LOG.debug("Initializing Servlet been registered");
             servletHandler.startServlet(); // this might throw ServletException, throw it to offending bundle.
 
             // Add the servletPath and the OSGiServletHandler to the backing map.
@@ -331,51 +333,95 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
         }
     }
 
-    protected void updateTrackedServletFilters(ServletFilterRegistration filterRegistration) {
-        List<Filter> filtersToRemove;
-        {
-            List<FilterProxy> allFilters = filterRegistration.getFilters();
-            filtersToRemove = new LinkedList<Filter>();
-            for (FilterProxy filterProxy : allFilters) {
-                filtersToRemove.add(filterProxy.getFilter());
-            }
-        }
-
-        for (Entry<String, HttpHandler> handlerEntry : mapper.getHttpHandlers()) {
-            // 1. Remove all possible tracked filters
-            // 2. Re-add them ordered
-            ((OSGiServletHandler) handlerEntry.getValue()).updateFilters(filtersToRemove, filterRegistration.getFilters(handlerEntry.getKey()));
-        }
-    }
-
-    protected void removeTrackedServletFilter(FilterProxy proxy) {
-        for (Entry<String, HttpHandler> handlerEntry : mapper.getHttpHandlers()) {
-            ((OSGiServletHandler) handlerEntry.getValue()).removeFilter(proxy.getFilter());
+    /**
+     * Add our set of initial Filters to the ServletHandler.
+     *
+     * @param context The associated HTTP context
+     * @throws ServletException
+     */
+    private void addInitialServletFilters(HttpContext context) throws ServletException {
+        for (FilterAndPath initialFilter : initialFilters) {
+            registerFilter(initialFilter.getFilter(), initialFilter.getPath(), null, context, null);
         }
     }
 
     /**
-     * Add our default set of Filters to the ServletHandler.
+     * Adds specified filter to all local HTTP contexts.
      *
-     * @param servletHandler The ServletHandler with the FilterChain
+     * @param filter The filter to add
+     * @param path The filter path
+     * @throws ServletException If registering specified filter fails
+     */
+    public void addServletFilter(Filter filter, String path) throws ServletException {
+        ReentrantLock lock = OSGiCleanMapper.getLock();
+        lock.lock();
+        try {
+            for (HttpContext context : mapper.httpContextToServletContextMap.keySet()) {
+                registerFilter(filter, path, null, context, null);
+            }
+
+        } finally {
+            lock.unlock();
+        }
+    }
+
+    /**
+     * Removes specified filter from all local HTTP contexts.
+     *
+     * @param filter The filter to remove
+     */
+    public void removeServletFilter(Filter filter) {
+        ReentrantLock lock = OSGiCleanMapper.getLock();
+        lock.lock();
+        try {
+            unregisterFilter(filter);
+        } finally {
+            lock.unlock();
+        }
+    }
+
+    /**
+     *
+     * @param filter
+     * @param urlPattern
+     * @param initparams
+     * @param context
+     * @param httpService
+     * @throws NamespaceException
      * @throws ServletException
      */
-    private void addServletFilters(OSGiServletHandler servletHandler) throws ServletException {
-        // wrap it
-        servletHandler.addFilter(new WrappingFilter(), WrappingFilter.class.getName(), null);
+    public void registerFilter(final Filter filter,
+                               final String urlPattern,
+                               final Dictionary initparams,
+                               HttpContext context,
+                               final HttpService httpService)
+            throws ServletException {
 
-        // watch it
+        ReentrantLock lock = OSGiCleanMapper.getLock();
+        lock.lock();
         try {
-            servletHandler.addFilter(new RequestReportingFilter(), RequestReportingFilter.class.getName(), null);
-        } catch (OXException e) {
-            throw new ServletException(e);
-        }
 
-        // Add tracked ones
-        ServletFilterRegistration filterRegistration = ServletFilterRegistration.getInstance();
-        List<Filter> filters = filterRegistration.getFilters(servletHandler.getServletPath());
-        for (Filter filter : filters) {
-            servletHandler.addFilter(filter, filter.getClass().getName(), null);
+
+            if (context == null) {
+                LOG.debug("No HttpContext provided, creating default");
+                context = null == httpService ? new HttpContextImpl(bundle) : httpService.createDefaultHttpContext();
+            }
+
+            OSGiServletContext servletContext =
+                    mapper.getServletContext(context);
+            if (servletContext == null) {
+                mapper.addContext(context, null);
+                servletContext = mapper.getServletContext(context);
+            }
+
+            FilterRegistration registration =
+                    servletContext.addFilter(Integer.toString(filter.hashCode()), filter);
+            registration.addMappingForUrlPatterns(null, urlPattern);
+
+            filter.init(new OSGiFilterConfig(servletContext));
+
+        } finally {
+            lock.unlock();
         }
     }
 
@@ -384,13 +430,15 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
      * <p/>
      * Keeps truck of all registrations, takes care of thread safety.
      *
-     * @param alias Alias to register, if wrong value than throws {@link NamespaceException}.
-     * @param context OSGi {@link HttpContext}, provides mime handling, security and bundle specific resource access.
+     * @param alias          Alias to register, if wrong value than throws {@link NamespaceException}.
+     * @param context        OSGi {@link HttpContext}, provides mime handling, security and bundle specific resource access.
      * @param internalPrefix Prefix to map request for this alias to.
      * @param httpService Used to {@link HttpService#createDefaultHttpContext()} if needed.
      * @throws NamespaceException If alias was invalid or already registered.
      */
-    public void registerResourceHandler(String alias, HttpContext context, String internalPrefix, HttpService httpService) throws NamespaceException {
+    public void registerResourceHandler(String alias, HttpContext context, String internalPrefix,
+                                        HttpService httpService)
+            throws NamespaceException {
 
         ReentrantLock lock = OSGiCleanMapper.getLock();
         lock.lock();
@@ -399,13 +447,19 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
 
             if (context == null) {
                 LOG.debug("No HttpContext provided, creating default");
-                context = httpService.createDefaultHttpContext();
+                context = null == httpService ? new HttpContextImpl(bundle) : httpService.createDefaultHttpContext();
             }
+
             if (internalPrefix == null) {
                 internalPrefix = "";
             }
+            OSGiServletContext servletContext = mapper.getServletContext(context);
 
-            mapper.addHttpHandler(alias, new OSGiResourceHandler(alias, internalPrefix, context));
+            mapper.addHttpHandler(alias,
+                                  new OSGiResourceHandler(alias,
+                                                          internalPrefix,
+                                                          context,
+                                                          servletContext));
         } finally {
             lock.unlock();
         }
@@ -416,7 +470,7 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
      * <p/>
      * Keeps truck of all registrations, takes care of thread safety.
      *
-     * @param alias Alias to unregister, if not owning alias {@link IllegalArgumentException} is thrown.
+     * @param alias       Alias to unregister, if not owning alias {@link IllegalArgumentException} is thrown.
      * @throws IllegalArgumentException If alias was not registered by calling bundle.
      */
     public void unregisterAlias(String alias) {
@@ -427,7 +481,7 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
             if (mapper.isLocalyRegisteredAlias(alias)) {
                 mapper.doUnregister(alias, true);
             } else {
-                LOG.warn("Bundle: {} tried to unregister not owned alias '{}{}", bundle, alias, '\'');
+                LOG.warn("Bundle: {} tried to unregister not owned alias '{}'", bundle, alias);
                 throw new IllegalArgumentException(new StringBuilder(64).append("Alias '").append(alias).append(
                     "' was not registered by you.").toString());
             }
@@ -436,6 +490,18 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
         }
     }
 
+    public void unregisterFilter(final Filter filter) {
+        ReentrantLock lock = OSGiCleanMapper.getLock();
+        lock.lock();
+        try {
+            for (OSGiServletContext servletContext : mapper.httpContextToServletContextMap.values()) {
+                servletContext.unregisterFilter(filter);
+            }
+        } finally {
+            lock.unlock();
+        }
+    }
+
     /**
      * Unregisters all <code>alias</code>es registered by owning bundle.
      */
@@ -445,10 +511,14 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
         ReentrantLock lock = OSGiCleanMapper.getLock();
         lock.lock();
         try {
-            for (String alias : mapper.getLocalAliases()) {
+            for (String alias : new LinkedHashSet<>(mapper.getLocalAliases())) {
                 LOG.debug("Unregistering '{}'", alias);
                 // remember not to call Servlet.destroy() owning bundle might be stopped already.
                 mapper.doUnregister(alias, false);
+                for (OSGiServletContext servletContext : mapper.httpContextToServletContextMap.values()) {
+                    servletContext.unregisterAllFilters();
+                }
+                mapper.httpContextToServletContextMap.clear();
             }
         } finally {
             lock.unlock();
@@ -456,7 +526,8 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
     }
 
     /**
-     * Part of Shutdown sequence. Unregister and clean up.
+     * Part of Shutdown sequence.
+     * Unregister and clean up.
      */
     public void unregisterAll() {
         LOG.info("Unregistering all registered aliases");
@@ -533,42 +604,41 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
     /**
      * Check if <code>servlet</code> has been already registered.
      * <p/>
-     * An instance of {@link Servlet} can be registed only once, so in case of servlet been registered before will throw
-     * {@link ServletException} as specified in OSGI HttpService Spec.
+     * An instance of {@link Servlet} can be registered only once, so in case of servlet been registered before will throw
+     * {@link ServletException} as specified in OSGi HttpService Spec.
      *
      * @param servlet {@link Servlet} to check if can be registered.
-     * @param servletPath the path under which the servlet should be registered
      * @throws ServletException Iff <code>servlet</code> has been registered before.
      */
-    private void validateServlet4RegOk(Servlet servlet, String servletPath) throws ServletException {
+    private void validateServlet4RegOk(Servlet servlet) throws ServletException {
         if (OSGiCleanMapper.containsServlet(servlet)) {
-
-            String msg = new StringBuilder(64).append("Servlet: '").append(servlet).append("', already registered.").append(
-                "\n Tried to register under path:").append(servletPath).toString();
+            String msg = "Servlet: '" + servlet + "', already registered.";
             LOG.warn(msg);
             throw new ServletException(msg);
         }
-        throw new UnsupportedOperationException("go implement validation");
     }
 
     /**
      * Looks up {@link OSGiServletHandler}.
      * <p/>
-     * If is already registered for <code>httpContext</code> then create new instance based on already registered. Else Create new one.
+     * If is already registered for <code>httpContext</code> then create new instance based on already registered. Else
+     * Create new one.
      * <p/>
      *
-     * @param servlet {@link Servlet} been registered.
+     * @param servlet     {@link Servlet} been registered.
      * @param httpContext {@link HttpContext} used for registration.
-     * @param initparams Init parameters that will be visible in {@link javax.servlet.ServletContext}.
+     * @param initparams  Init parameters that will be visible in {@link javax.servlet.ServletContext}.
      * @return Found or created {@link OSGiServletHandler}.
      */
-    private OSGiServletHandler findOrCreateOSGiServletHandler(Servlet servlet, HttpContext httpContext, Dictionary initparams) {
+    private OSGiServletHandler findOrCreateOSGiServletHandler(
+            Servlet servlet, HttpContext httpContext, Dictionary initparams) {
         OSGiServletHandler osgiServletHandler;
 
-        if (mapper.containsContext(httpContext)) {
+        List<OSGiServletHandler> servletHandlers =
+                mapper.getContext(httpContext);
+        if (servletHandlers != null) {
             LOG.debug("Reusing ServletHandler");
             // new servlet handler for same configuration, different servlet and alias
-            List<OSGiServletHandler> servletHandlers = mapper.getContext(httpContext);
             osgiServletHandler = servletHandlers.get(0).newServletHandler(servlet);
             servletHandlers.add(osgiServletHandler);
         } else {
@@ -584,25 +654,39 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
             } else {
                 params = new HashMap<String, String>(0);
             }
-            osgiServletHandler = new OSGiServletHandler(servlet, httpContext, params);
-            ArrayList<OSGiServletHandler> servletHandlers = new ArrayList<OSGiServletHandler>(1);
+
+            servletHandlers = new ArrayList<OSGiServletHandler>(1);
+            mapper.addContext(httpContext,
+                    mapper.getServletContext(httpContext),
+                    servletHandlers);
+
+            final OSGiServletContext servletContext =
+                    mapper.getServletContext(httpContext);
+
+            assert servletContext != null;
+
+            osgiServletHandler =
+                    new OSGiServletHandler(servlet,
+                                           httpContext,
+                                           servletContext,
+                                           params);
             servletHandlers.add(osgiServletHandler);
-            mapper.addContext(httpContext, servletHandlers);
+            osgiServletHandler.setFilterChainFactory(servletContext.getFilterChainFactory());
         }
-        osgiServletHandler.addFilter(new OSGiAuthFilter(httpContext), "AuthorisationFilter", Collections.<String, String> emptyMap());
+
         return osgiServletHandler;
     }
 
-    /*
-     * alias = /pathTo/theServlet
-     * originalAlias = /pathTo/theServlet/additional/path/info
-     */
-    private void updateMappingInfo(final Request request, final String alias, final String originalAlias) {
-        final MappingData mappingData = request.obtainMappingData();
-        //Change contextPath from "/" to the empty Sring for the default context in the httpservice
-        mappingData.contextPath.setString("");
+    private void updateMappingInfo(final Request request,
+            final String alias, final String originalAlias) {
 
-        mappingData.wrapperPath.setString(alias);
+        final MappingData mappingData = request.obtainMappingData();
+        mappingData.contextPath.setString("");
+        if (alias.equals("/")) {
+            mappingData.wrapperPath.setString("");
+        } else {
+            mappingData.wrapperPath.setString(alias);
+        }
 
         if (alias.length() != originalAlias.length()) {
             String pathInfo = originalAlias.substring(alias.length());
@@ -617,19 +701,23 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
     }
 
     @Override
-    protected void customizedErrorPage(Request req, Response res)
-        throws Exception {
+    protected ErrorPageGenerator getErrorPageGenerator(Request request) {
+        return errorPageGenerator;
+    }
 
+    private void writeErrorPage(Request req, Response res) throws Exception {
         res.setStatus(HttpStatus.NOT_FOUND_404);
         final ByteBuffer bb = getErrorPage(404, "Not found", "Resource does not exist.");
         res.setContentLength(bb.limit());
         res.setContentType("text/html");
-        OutputBuffer out = res.getOutputBuffer();
+        org.glassfish.grizzly.http.io.OutputBuffer out = res.getOutputBuffer();
         out.prepareCharacterEncoder();
         out.write(bb.array(), bb.arrayOffset() + bb.position(), bb.remaining());
         out.close();
     }
 
+    private static final CharsetEncoder encoder = Charset.forName("UTF-8").newEncoder();
+
     /**
      * Generate an error page based on our servlet defaults.
      *
@@ -637,13 +725,34 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
      */
     public synchronized static ByteBuffer getErrorPage(int code,
         String message, String description) throws IOException {
-
         String body = com.openexchange.tools.servlet.http.Tools.getErrorPage(
             code, message, description);
+        CharBuffer reponseBuffer = CharBuffer.allocate(4096);
         reponseBuffer.clear();
         reponseBuffer.put(body);
         reponseBuffer.flip();
         return encoder.encode(reponseBuffer);
 
     }
+
+    // -----------------------------------------------------------------------------------------
+
+    private static final OXThreadMarker DUMMY = new OXThreadMarker() {
+
+        @Override
+        public void setHttpRequestProcessing(boolean httpProcessing) {
+            // Nothing
+        }
+
+        @Override
+        public boolean isHttpRequestProcessing() {
+            return false;
+        }
+    };
+
+    private static OXThreadMarker threadMarker() {
+        Thread t = Thread.currentThread();
+        return t instanceof OXThreadMarker ? (OXThreadMarker) t : DUMMY;
+    }
+
 }
