@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -78,11 +78,11 @@ public class RequestRegistryEntry implements Comparable<RequestRegistryEntry> {
      * @param thread the thread associated with the request
      * @param propertyMap
      */
-    public RequestRegistryEntry(final long number, final HttpServletRequest request, final HttpServletResponse response, final Thread thread, Map<String, String> propertyMap) {
+    public RequestRegistryEntry(long number, HttpServletRequest request, HttpServletResponse response, Thread thread,  Map<String, String> propertyMap) {
         super();
         this.number = number;
         this.thread = thread;
-        this.propertyMap = null == propertyMap ? Collections.<String, String> emptyMap() : propertyMap;
+        this.propertyMap = null == propertyMap ? Collections.<String, String> emptyMap() : propertyMap;;
         this.request = request;
         this.response = response;
         this.birthTime = System.currentTimeMillis();
